//+------------------------------------------------------------------+
//|                                                     RandFile.mq4 |
//|                                                          Dubovik |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Dubovik"
#property link      ""
#property version   "1.00"
#property strict
#property script_show_inputs


//--- input parameters
input string S1 = "   ";
input int      Millions = 1;
input bool     CalcSeries = false;


const int MillionsMultiplier = 1000000;




void OnStart()
{
	//  
	MathSrand(GetTickCount());

	//          
	WriteFiles();
}




void WriteFiles()
{
	//    
	int iRandFile = FileOpen("RandFile.bin", FILE_BIN | FILE_WRITE);
	int iStatFile = FileOpen("RandStat.csv", FILE_CSV | FILE_WRITE, "\t");
	int iOnesSeriesFile = FileOpen("RandOnesSeries.csv", FILE_CSV | FILE_WRITE, "\t");
	int iZerosSeriesFile = FileOpen("RandZerosSeries.csv", FILE_CSV | FILE_WRITE, "\t");

		//    ...
		if ( (iRandFile != INVALID_HANDLE) &&
						(iStatFile != INVALID_HANDLE) &&
						(iOnesSeriesFile != INVALID_HANDLE) &&
						(iZerosSeriesFile != INVALID_HANDLE) )
		{
			//    
			uint uiRand;

			// ,           
			//   
			int iRandBytesTable [256] = {0};

			//         
			long lOnesSeries[];
			long lZerosSeries[];
			//   
			uint iLastBit = 0;
			//   
			int iBitsCount = 0;

			//   
			uint iStart = GetTickCount();

			int iSize = Millions * MillionsMultiplier;
				//         
				for (int i = 0; i < iSize; i++)
				{
					uiRand = Rand32();

					FileWriteInteger(iRandFile, uiRand);

					//          
					iRandBytesTable[uiRand & 0xff]++;
					iRandBytesTable[(uiRand & 0xff00) >> 8]++;
					iRandBytesTable[(uiRand & 0xff0000) >> 16]++;
					iRandBytesTable[(uiRand & 0xff000000) >> 24]++;

						if (CalcSeries)
							BitSeries(uiRand, lOnesSeries, lZerosSeries, iLastBit, iBitsCount);
				}

			Print(" ", Millions * 4, " .    ", GetTickCount() - iStart, " .");

			//          
			long lTotalOneBitsRand = 0;

			//     
			int iBits;
				for (int i = 0; i < 256; i++)
				{
					iBits = 0;
						// ,        i
						for (int j = 0; j < 8; j++)
						{
							iBits += (i >> j) & 1;
						}

					//   ,    
					lTotalOneBitsRand += iBits * iRandBytesTable[i];

					//     
					FileWrite(iStatFile, i, iRandBytesTable[i]);
				}

			//      
			Print("  : ", lTotalOneBitsRand);


				for (int i = 0; i < ArraySize(lOnesSeries); i++)
				{
					FileWrite(iOnesSeriesFile, i, lOnesSeries[i]);
				}
				for (int i = 0; i < ArraySize(lZerosSeries); i++)
				{
					FileWrite(iZerosSeriesFile, i, lZerosSeries[i]);
				}

			//  
			FileClose(iRandFile);
			FileClose(iStatFile);
			FileClose(iOnesSeriesFile);
			FileClose(iZerosSeriesFile);
		}
		else
		{
			Print("   ");
		}
}




//      
void BitSeries(uint a_uiRand, long& a_lOnesSeries[], long& a_lZerosSeries[], uint& a_uiLastBit, uint& a_uiBitsCount)
{
	uint uiCurrentBit;

		for (int i = 31; i >= 0; i--)
		{
			uiCurrentBit = (a_uiRand >> i) & 1;

				if (uiCurrentBit == a_uiLastBit)
				{
					a_uiBitsCount++;
				}
				else
				{
						if (a_uiLastBit == 0)
						{
							WriteValue(a_lZerosSeries, a_uiBitsCount);
						}
						else
						{
							WriteValue(a_lOnesSeries, a_uiBitsCount);
						}
					a_uiBitsCount = 0;
					a_uiLastBit = uiCurrentBit;
				}
		}
}




//    .      .
void WriteValue(long& a_lSeries[], uint a_uiValue)
{
		//    
		if ((uint)ArraySize(a_lSeries) <= a_uiValue)
		{
			int iOldSize = ArraySize(a_lSeries);
			ArrayResize(a_lSeries, a_uiValue + 1, 1000);
				//   .
				//     ,       
				for (int i = iOldSize; i < ArraySize(a_lSeries); i++)
				{
					a_lSeries[i] = 0;
				}
		}

	//  
	a_lSeries[a_uiValue]++;
}




//    - 32     
uint Rand32()
{
	int iResult = 0;
	iResult |= MathRand();
	iResult |= MathRand() << 15;
	iResult |= (MathRand() & 0x3) << 30;
	return iResult;
}
